@@ -1,5 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013,2014 Eric Bavier <bavier@member.fsf.org>
+;;; Copyright © 2013, 2014 Eric Bavier <bavier@member.fsf.org>
+;;; Copyright © 2015, 2016 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2018, 2019 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -23,38 +25,43 @@
   #:use-module (guix build-system gnu)
   #:use-module (gnu packages)
   #:use-module (gnu packages perl)
-  #:use-module (gnu packages which)
+  #:use-module (gnu packages pkg-config)
+  #:use-module (gnu packages base)
   #:use-module (gnu packages guile))
 
 (define-public autogen
   (package
     (name "autogen")
-    (version "5.18.4")
+    (version "5.18.16")
     (source
      (origin
-      (method url-fetch)
-      (uri (string-append "mirror://gnu/autogen/rel"
-                          version "/autogen-"
-                          version ".tar.gz"))
-      (sha256
-       (base32
-        "0pbjzwgvmjvi6nl1bcyhfc9kl93s6321dgmvp5dqdip7i8dgin9w"))))
+       (method url-fetch)
+       (uri (string-append "mirror://gnu/autogen/rel" version
+                           "/autogen-" version ".tar.xz"))
+       (sha256
+        (base32 "16mlbdys8q4ckxlvxyhwkdnh1ay9f6g0cyp1kylkpalgnik398gq"))))
     (build-system gnu-build-system)
-    (native-inputs `(("perl" ,perl)))   ;for doc generator mdoc
-    (inputs `(("which" ,which)
-              ("guile" ,guile-2.0)))
+    (native-inputs `(("pkg-config" ,pkg-config)
+                     ("which" ,which)))
+    (inputs `(("guile" ,guile-2.2)
+              ("perl" ,perl)))          ; for doc generator mdoc
     (arguments
-     '(#:phases (alist-cons-before
-                 'patch-source-shebangs 'patch-test-scripts
-                 (lambda _
-                   (let ((sh (which "sh")))
-                     (substitute*
-                         (append (find-files "agen5/test" "\\.test$")
-                                 (find-files "autoopts/test" "\\.(test|in)$"))
-                       (("/bin/sh") sh)
-                       (("/usr/bin/tr") "tr"))))
-                 %standard-phases)))
-    (home-page "http://www.gnu.org/software/autogen/")
+     '(#:configure-flags
+       ;; XXX Needed to build 5.18.16.  ./configure fails without it:
+       ;; “Something went wrong bootstrapping makefile fragments for
+       ;;  automatic dependency tracking.  Try re-running configure with […]”
+       (list "--disable-dependency-tracking")
+       #:phases
+       (modify-phases %standard-phases
+         (add-before 'patch-source-shebangs 'patch-test-scripts
+           (lambda _
+             (let ((sh (which "sh")))
+               (substitute*
+                   (append (find-files "agen5/test" "\\.test$")
+                           (find-files "autoopts/test" "\\.(test|in)$"))
+                 (("/bin/sh") sh))
+               #t))))))
+    (home-page "https://www.gnu.org/software/autogen/")
     (synopsis "Automated program generator")
     (description
      "AutoGen is a program to ease the maintenance of programs that contain
