@@ -1,14 +1,16 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2013, 2014, 2015, 2016, 2017 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2015, 2016, 2017, 2018 Mark H Weaver <mhw@netris.org>
-;;; Copyright © 2016, 2017 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2016, 2017, 2018. 2019 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2016, 2017 Ricardo Wurmus <rekado@elephly.net>
 ;;; Copyright © 2017 Alex Vong <alexvong1995@gmail.com>
 ;;; Copyright © 2017 Andy Patterson <ajpatter@uwaterloo.ca>
-;;; Copyright © 2017, 2018 Rutger Helling <rhelling@mykolab.com>
-;;; Copyright © 2017, 2018 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2017, 2018, 2019 Rutger Helling <rhelling@mykolab.com>
+;;; Copyright © 2017, 2018, 2019 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;; Copyright © 2018 Danny Milosavljevic <dannym@scratchpost.org>
 ;;; Copyright © 2018 Sou Bunnbu <iyzsong@member.fsf.org>
+;;; Copyright © 2018 Julien Lepiller <julien@lepiller.eu>
+;;; Copyright © 2019 Guy Fleury Iteriteka <hoonandon@gmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -28,19 +30,25 @@
 (define-module (gnu packages virtualization)
   #:use-module (gnu packages)
   #:use-module (gnu packages admin)
+  #:use-module (gnu packages assembly)
   #:use-module (gnu packages attr)
   #:use-module (gnu packages autotools)
   #:use-module (gnu packages bison)
   #:use-module (gnu packages check)
+  #:use-module (gnu packages cmake)
   #:use-module (gnu packages compression)
+  #:use-module (gnu packages cross-base)
   #:use-module (gnu packages curl)
   #:use-module (gnu packages cyrus-sasl)
   #:use-module (gnu packages disk)
   #:use-module (gnu packages dns)
   #:use-module (gnu packages docbook)
   #:use-module (gnu packages documentation)
+  #:use-module (gnu packages figlet)
+  #:use-module (gnu packages firmware)
   #:use-module (gnu packages flex)
   #:use-module (gnu packages fontutils)
+  #:use-module (gnu packages freedesktop)
   #:use-module (gnu packages gettext)
   #:use-module (gnu packages gl)
   #:use-module (gnu packages glib)
@@ -61,6 +69,7 @@
   #:use-module (gnu packages protobuf)
   #:use-module (gnu packages python)
   #:use-module (gnu packages python-web)
+  #:use-module (gnu packages python-xyz)
   #:use-module (gnu packages pulseaudio)
   #:use-module (gnu packages selinux)
   #:use-module (gnu packages sdl)
@@ -69,15 +78,17 @@
   #:use-module (gnu packages textutils)
   #:use-module (gnu packages tls)
   #:use-module (gnu packages web)
+  #:use-module (gnu packages wget)
   #:use-module (gnu packages xdisorg)
   #:use-module (gnu packages xml)
+  #:use-module (gnu packages xorg)
+  #:use-module (guix build-system cmake)
   #:use-module (guix build-system gnu)
   #:use-module (guix build-system go)
   #:use-module (guix build-system python)
   #:use-module (guix download)
   #:use-module (guix git-download)
-  #:use-module ((guix licenses) #:select (gpl2 gpl2+ gpl3+ lgpl2.1 lgpl2.1+
-                                               asl2.0))
+  #:use-module ((guix licenses) #:prefix license:)
   #:use-module (guix packages)
   #:use-module (guix utils)
   #:use-module (srfi srfi-1))
@@ -95,15 +106,16 @@
 (define-public qemu
   (package
     (name "qemu")
-    (version "2.12.0")
+    (version "3.1.0")
     (source (origin
              (method url-fetch)
              (uri (string-append "https://download.qemu.org/qemu-"
                                  version ".tar.xz"))
-             (patches (search-patches "qemu-CVE-2018-11806.patch"))
+             (patches (search-patches "qemu-CVE-2018-16872.patch"
+                                      "qemu-CVE-2019-6778.patch"))
              (sha256
               (base32
-               "1z66spkm1prvhbq7h5mfnp0i6mmamsb938fqmdfvyrgzc7rh34z6"))))
+               "1z5bd5nfyjvhfi1s95labc82y4hjdjjkdabw931362ls0zghh1ba"))))
     (build-system gnu-build-system)
     (arguments
      '(;; Running tests in parallel can occasionally lead to failures, like:
@@ -227,7 +239,7 @@ the KVM kernel module in Linux.  When using KVM, QEMU can virtualize x86,
 server and embedded PowerPC, and S390 guests.")
 
     ;; Many files are GPLv2+, but some are GPLv2-only---e.g., `memory.c'.
-    (license gpl2)
+    (license license:gpl2)
 
     ;; Several tests fail on MIPS; see <http://hydra.gnu.org/build/117914>.
     (supported-systems (delete "mips64el-linux" %supported-systems))))
@@ -337,12 +349,12 @@ system on a hypervisor.  Via GObject Introspection, the API is available in
 all common programming languages.  Vala bindings are also provided.")
     ;; The library files are released under LGPLv2.1 or later; the source
     ;; files in the "tools" directory are released under GPLv2+.
-    (license (list lgpl2.1+ gpl2+))))
+    (license (list license:lgpl2.1+ license:gpl2+))))
 
 (define-public lxc
   (package
     (name "lxc")
-    (version "3.0.1")
+    (version "3.1.0")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -350,7 +362,7 @@ all common programming languages.  Vala bindings are also provided.")
                     version ".tar.gz"))
               (sha256
                (base32
-                "1nyml98k28sc5sda0260cmby4irkpnhpwgmx4yhqy10wpr4nr625"))))
+                "1igxqgx8q9cp15mcp1y8j564bl85ijw04jcmgb1s5bmfbg1751sd"))))
     (build-system gnu-build-system)
     (native-inputs
      `(("pkg-config" ,pkg-config)))
@@ -360,9 +372,11 @@ all common programming languages.  Vala bindings are also provided.")
        ("libseccomp" ,libseccomp)
        ("libselinux" ,libselinux)))
     (arguments
-     '(#:configure-flags
-       '("--sysconfdir=/etc"
-         "--localstatedir=/var")
+     `(#:configure-flags
+       (list (string-append "--docdir=" (assoc-ref %outputs "out")
+                            "/share/doc/" ,name "-" ,version)
+             "--sysconfdir=/etc"
+             "--localstatedir=/var")
        #:phases
        (modify-phases %standard-phases
          (replace 'install
@@ -382,19 +396,19 @@ all common programming languages.  Vala bindings are also provided.")
      "LXC is a userspace interface for the Linux kernel containment features.
 Through a powerful API and simple tools, it lets Linux users easily create and
 manage system or application containers.")
-    (license lgpl2.1+)))
+    (license license:lgpl2.1+)))
 
 (define-public libvirt
   (package
     (name "libvirt")
-    (version "4.3.0")
+    (version "4.10.0")
     (source (origin
               (method url-fetch)
               (uri (string-append "https://libvirt.org/sources/libvirt-"
                                   version ".tar.xz"))
               (sha256
                (base32
-                "1dy243dqaj174hcka0my7q781wf0dvyi7f9328nwnplqicnf4cd5"))))
+                "0v17zzyyb25nn9l18v5244myg7590dp6ppwgi8xysipifc0q77bz"))))
     (build-system gnu-build-system)
     (arguments
      `(;; FAIL: virshtest
@@ -438,6 +452,8 @@ manage system or application containers.")
                #t))))))
     (inputs
      `(("libxml2" ,libxml2)
+       ("eudev" ,eudev)
+       ("libpciaccess" ,libpciaccess)
        ("gnutls" ,gnutls)
        ("dbus" ,dbus)
        ("qemu" ,qemu)
@@ -467,7 +483,7 @@ manage system or application containers.")
 capabilities of recent versions of Linux.  The library aims at providing long
 term stable C API initially for the Xen paravirtualization but should be able
 to integrate other virtualization mechanisms if needed.")
-    (license lgpl2.1+)))
+    (license license:lgpl2.1+)))
 
 (define-public libvirt-glib
   (package
@@ -515,18 +531,18 @@ three libraries:
 @item libvirt-gobject - GObjects for managing libvirt objects
 @end enumerate
 ")
-    (license lgpl2.1+)))
+    (license license:lgpl2.1+)))
 
 (define-public python-libvirt
   (package
     (name "python-libvirt")
-    (version "4.1.0")
+    (version "4.10.0")
     (source (origin
               (method url-fetch)
               (uri (pypi-uri "libvirt-python" version))
               (sha256
                (base32
-                "1ixqhxjkczl8vk9wjx4cknw4374cw5nnsacbd2s755kpd0ys7hny"))))
+                "11fipj9naihgc9afc8bz5hi05xa1shp4qcy170sa18p3sl4zljb9"))))
     (build-system python-build-system)
     (arguments
      `(#:phases
@@ -550,7 +566,7 @@ three libraries:
     (synopsis "Python bindings to libvirt")
     (description "This package provides Python bindings to the libvirt
 virtualization library.")
-    (license lgpl2.1+)))
+    (license license:lgpl2.1+)))
 
 (define-public python2-libvirt
   (package-with-python2 python-libvirt))
@@ -558,7 +574,7 @@ virtualization library.")
 (define-public virt-manager
   (package
     (name "virt-manager")
-    (version "1.5.1")
+    (version "2.1.0")
     (source (origin
               (method url-fetch)
               (uri (string-append "https://virt-manager.org/download/sources"
@@ -566,11 +582,10 @@ virtualization library.")
                                   version ".tar.gz"))
               (sha256
                (base32
-                "1ardmd4sxdmd57y7qpka44gf09c1yq2g0xs074d3k1h925crv27f"))))
+                "1m038kyngmxlgz91c7z8g73lb2wy0ajyah871a3g3wb5cnd0dsil"))))
     (build-system python-build-system)
     (arguments
-     `(#:python ,python-2
-       #:use-setuptools? #f ; Uses custom distutils 'install' command.
+     `(#:use-setuptools? #f          ; uses custom distutils 'install' command
        ;; Some of the tests seem to require network access to install virtual
        ;; machines.
        #:tests? #f
@@ -590,7 +605,7 @@ virtualization library.")
              #t))
          (add-after 'unpack 'fix-default-uri
            (lambda* (#:key inputs #:allow-other-keys)
-             ;; xen is not available for now - so only patch qemu
+             ;; Xen is not available for now - so only patch qemu.
              (substitute* "virtManager/connect.py"
                (("/usr(/bin/qemu-system)" _ suffix)
                 (string-append (assoc-ref inputs "qemu") suffix)))
@@ -627,18 +642,18 @@ virtualization library.")
        ("libosinfo" ,libosinfo)
        ("vte" ,vte)
        ("gobject-introspection" ,gobject-introspection)
-       ("python2-libvirt" ,python2-libvirt)
-       ("python2-requests" ,python2-requests)
-       ("python2-ipaddr" ,python2-ipaddr)
-       ("python2-pycairo" ,python2-pycairo)
-       ("python2-pygobject" ,python2-pygobject)
-       ("python2-libxml2" ,python2-libxml2)
+       ("python-libvirt" ,python-libvirt)
+       ("python-requests" ,python-requests)
+       ("python-ipaddress" ,python-ipaddress)
+       ("python-pycairo" ,python-pycairo)
+       ("python-pygobject" ,python-pygobject)
+       ("python-libxml2" ,python-libxml2)
        ("spice-gtk" ,spice-gtk)))
     ;; virt-manager searches for qemu-img or kvm-img in the PATH.
     (propagated-inputs
      `(("qemu" ,qemu)))
     (native-inputs
-     `(("glib" ,glib "bin")             ; glib-compile-schemas.
+     `(("glib" ,glib "bin")             ; glib-compile-schemas
        ("gtk+" ,gtk+ "bin")             ; gtk-update-icon-cache
        ("perl" ,perl)                   ; pod2man
        ("intltool" ,intltool)))
@@ -649,19 +664,19 @@ virtualization library.")
 virtual machines through libvirt.  It primarily targets KVM VMs, but also
 manages Xen and LXC (Linux containers).  It presents a summary view of running
 domains, their live performance and resource utilization statistics.")
-    (license gpl2+)))
+    (license license:gpl2+)))
 
 (define-public criu
   (package
     (name "criu")
-    (version "3.7")
+    (version "3.11")
     (source (origin
               (method url-fetch)
               (uri (string-append "http://download.openvz.org/criu/criu-"
                                   version ".tar.bz2"))
               (sha256
                (base32
-                "0qrpz7pvnks34v7d8lb73flz3mb7qwnib94pdwaxh0mskn8470fq"))))
+                "03nimyn3wy5mlw30gq7bvlzvvprqjv8f25240yj5arzlld8mhsw8"))))
     (build-system gnu-build-system)
     (arguments
      `(#:test-target "test"
@@ -690,7 +705,23 @@ domains, their live performance and resource utilization statistics.")
                                (assoc-ref inputs "docbook-xsl")
                                "/xml/xsl/docbook-xsl-"
                                ,(package-version docbook-xsl)
-                               "/manpages/docbook.xsl")))
+                               "/manpages/docbook.xsl"))
+               (("\\$\\(XMLTO\\);")
+                (string-append (assoc-ref inputs "xmlto")
+                               "/bin/xmlto;")))
+             #t))
+         (add-after 'unpack 'hardcode-variables
+           (lambda* (#:key inputs #:allow-other-keys)
+             ;; Hardcode arm version detection
+             (substitute* "Makefile"
+               (("ARMV.*:=.*") "ARMV := 7\n"))
+             ;; We are currently using python-2
+             (substitute* "crit/Makefile"
+               (("\\$\\(PYTHON\\)") "python2"))
+             (substitute* "lib/Makefile"
+               (("\\$\\(PYTHON\\)")
+                (string-append (assoc-ref inputs "python")
+                               "/bin/python")))
              #t))
          (add-before 'build 'fix-symlink
            (lambda* (#:key inputs #:allow-other-keys)
@@ -743,7 +774,7 @@ was frozen at.  The distinctive feature of the CRIU project is that it is
 mainly implemented in user space.")
     ;; The project is licensed under GPLv2; files in the lib/ directory are
     ;; LGPLv2.1.
-    (license (list gpl2 lgpl2.1))))
+    (license (list license:gpl2 license:lgpl2.1))))
 
 (define-public qmpbackup
   (package
@@ -765,68 +796,76 @@ mainly implemented in user space.")
     (description "qmpbackup is designed to create and restore full and
 incremental backups of running QEMU virtual machines via QMP, the QEMU
 Machine Protocol.")
-    (license gpl3+)))
+    (license license:gpl3+)))
 
-(define-public lookingglass
-  (package
-   (name "lookingglass")
-   (version "a11")
-   (source
-    (origin
-     (method url-fetch)
-     (uri (string-append "https://github.com/gnif/LookingGlass/archive/"
-                         version ".tar.gz"))
-     (file-name (string-append name "-" version))
-     (sha256
-      (base32
-       "11qwyp332l66sqksqa0z9439yi4accmbq7wjc6kikc5fimdh9wk5"))))
-   (build-system gnu-build-system)
-   (inputs `(("fontconfig" ,fontconfig)
-             ("glu" ,glu)
-             ("mesa" ,mesa)
-             ("openssl" ,openssl)
-             ("sdl2" ,sdl2)
-             ("sdl2-ttf" ,sdl2-ttf)
-             ("spice-protocol" ,spice-protocol)))
-   (native-inputs `(("libconfig" ,libconfig)
-                    ("nettle" ,nettle)
-                    ("pkg-config" ,pkg-config)))
-   (arguments
-    `(#:tests? #f ;; No tests are available.
-      #:make-flags '("CC=gcc")
-      #:phases (modify-phases %standard-phases
-                 (replace 'configure
-                   (lambda* (#:key outputs #:allow-other-keys)
-                     (chdir "client")
-                     #t))
-                 (replace 'install
-                   (lambda* (#:key outputs #:allow-other-keys)
-                     (install-file "bin/looking-glass-client"
-                                   (string-append (assoc-ref outputs "out")
-                                                  "/bin"))
-                     #t)))))
-   (home-page "https://looking-glass.hostfission.com")
-   (synopsis "KVM Frame Relay (KVMFR) implementation")
-   (description "Looking Glass allows the use of a KVM (Kernel-based Virtual
+(define-public looking-glass-client
+  (let ((commit "182c4752d57690da7f99d5e788de9b8baea33895"))
+    (package
+     (name "looking-glass-client")
+     (version (string-append "a12-" (string-take commit 7)))
+     (source
+      (origin
+       (method git-fetch)
+       (uri (git-reference (url "https://github.com/gnif/LookingGlass")
+                           (commit commit)))
+       (file-name (git-file-name name version))
+       (sha256
+        (base32
+         "02bq46ndmzq9cihazzn7xq1x7q5nzm7iw4l9lqzihxcxp9famkhw"))))
+     (build-system cmake-build-system)
+     (inputs `(("fontconfig" ,fontconfig)
+               ("glu" ,glu)
+               ("mesa" ,mesa)
+               ("openssl" ,openssl)
+               ("sdl2" ,sdl2)
+               ("sdl2-ttf" ,sdl2-ttf)
+               ("spice-protocol" ,spice-protocol)
+               ("wayland" ,wayland)))
+     (native-inputs `(("libconfig" ,libconfig)
+                      ("nettle" ,nettle)
+                      ("pkg-config" ,pkg-config)))
+     (arguments
+      `(#:tests? #f ;; No tests are available.
+        #:make-flags '("CC=gcc")
+        #:phases (modify-phases %standard-phases
+                   (add-before 'configure 'chdir-to-client
+                     (lambda* (#:key outputs #:allow-other-keys)
+                       (chdir "client")
+                       #t))
+                   (replace 'install
+                     (lambda* (#:key outputs #:allow-other-keys)
+                       (install-file "looking-glass-client"
+                                     (string-append (assoc-ref outputs "out")
+                                                    "/bin"))
+                       #t))
+                   )))
+     (home-page "https://looking-glass.hostfission.com")
+     (synopsis "KVM Frame Relay (KVMFR) implementation")
+     (description "Looking Glass allows the use of a KVM (Kernel-based Virtual
 Machine) configured for VGA PCI Pass-through without an attached physical
 monitor, keyboard or mouse.  It displays the VM's rendered contents on your main
 monitor/GPU.")
-   ;; This package requires SSE instructions.
-   (supported-systems '("i686-linux" "x86_64-linux"))
-   (license gpl2+)))
+     ;; This package requires SSE instructions.
+     (supported-systems '("i686-linux" "x86_64-linux"))
+     (license license:gpl2+))))
+
+(define-public lookingglass
+  (deprecated-package "lookingglass" looking-glass-client))
 
 (define-public runc
   (package
     (name "runc")
-    (version "1.0.0-rc5")
+    (version "1.0.0-rc6")
     (source (origin
               (method url-fetch)
               (uri (string-append
                     "https://github.com/opencontainers/runc/releases/"
                     "download/v" version "/runc.tar.xz"))
+              (file-name (string-append name "-" version ".tar.xz"))
+              (patches (search-patches "runc-CVE-2019-5736.patch"))
               (sha256
                (base32
-                "081avdzwnqpk368wbaihlzsypaxpj42d7699h7jgp0fks14x4103"))))
+                "1c7832dq70slkjh8qp2civ1wxhhdd2hrx84pq7db1mmqc9fdr3cc"))))
     (build-system go-build-system)
     (arguments
      '(#:import-path "github.com/opencontainers/runc"
@@ -870,12 +909,12 @@ packaged according to the
 @uref{https://github.com/opencontainers/runtime-spec/blob/master/spec.md, Open
 Container Initiative (OCI) format} and is a compliant implementation of the
 Open Container Initiative specification.")
-    (license asl2.0)))
+    (license license:asl2.0)))
 
 (define-public umoci
   (package
     (name "umoci")
-    (version "0.4.0")
+    (version "0.4.4")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -884,7 +923,7 @@ Open Container Initiative specification.")
               (file-name (string-append "umoci-" version ".tar.xz"))
               (sha256
                (base32
-                "0hg7hs4dagj2fgymm4b4s68k1v2k2093s3jg0d94j0ixhfmyg9nd"))))
+                "1wchmha5k2f370jfijmx9fqp0cp99zfa9ajmfbq3j24qc8p5k8lk"))))
     (build-system go-build-system)
     (arguments
      '(#:import-path "github.com/openSUSE/umoci"
@@ -915,7 +954,7 @@ Open Container Initiative specification.")
     (description
      "@command{umoci} is a tool that allows for high-level modification of an
 Open Container Initiative (OCI) image layout and its tagged images.")
-    (license asl2.0)))
+    (license license:asl2.0)))
 
 (define-public skopeo
   (package
@@ -945,6 +984,7 @@ Open Container Initiative (OCI) image layout and its tagged images.")
     (arguments
      '(#:import-path "github.com/projectatomic/skopeo"
        #:install-source? #f
+       #:tests? #f ; The tests require Docker
        #:phases
        (modify-phases %standard-phases
          (replace 'build
@@ -977,4 +1017,311 @@ the image.
 @item Delete container images from a remote container registry.
 
 @end enumerate")
-    (license asl2.0)))
+    (license license:asl2.0)))
+
+(define-public python-vagrant
+  (package
+    (name "python-vagrant")
+    (version "0.5.15")
+    (source
+      (origin
+        (method url-fetch)
+        (uri (pypi-uri "python-vagrant" version))
+        (sha256
+         (base32
+          "1ikrh6canhcxg5y7pzmkcnnydikppv7s6sm9prfx90nk0ac8m6mg"))))
+    (build-system python-build-system)
+    (arguments
+     '(#:tests? #f)) ; tests involve running vagrant.
+    (home-page "https://github.com/todddeluca/python-vagrant")
+    (synopsis "Python bindings for Vagrant")
+    (description
+     "Python-vagrant is a Python module that provides a thin wrapper around the
+@code{vagrant} command line executable, allowing programmatic control of Vagrant
+virtual machines.")
+    (license license:expat)))
+
+(define-public bubblewrap
+  (package
+    (name "bubblewrap")
+    (version "0.3.1")
+    (source (origin
+              (method url-fetch)
+              (uri (string-append "https://github.com/projectatomic/bubblewrap/"
+                                  "releases/download/v" version "/bubblewrap-"
+                                  version ".tar.xz"))
+              (sha256
+               (base32
+                "1y2bdlxnlr84xcbf31lzirc292c5ak9bd2wvcvh4ppsliih6pjny"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:phases
+       (modify-phases %standard-phases
+         (add-after 'unpack 'fix-test
+           (lambda* (#:key outputs #:allow-other-keys)
+             ;; Tests try to access /var/tmp, which is not possible in our build
+             ;; environment.  Let's give them another directory.
+             ;; /tmp gets overriden in some tests, so we need another directory.
+             ;; the only possibility is the output directory.
+             (let ((tmp-dir (string-append (assoc-ref outputs "out") "/tmp")))
+               (mkdir-p tmp-dir)
+               (substitute* "tests/test-run.sh"
+                 (("/var/tmp") tmp-dir)
+                 ;; Tests create a temporary python script, so fix its shebang.
+                 (("/usr/bin/env python") (which "python"))
+                 ;; Some tests try to access /usr, but that doesn't exist.
+                 ;; Give them /gnu instead.
+                 (("/usr") "/gnu")
+                 (("  */bin/bash") (which "bash"))
+                 (("/bin/sh") (which "sh"))
+                 (("findmnt") (which "findmnt"))))
+             #t))
+         ;; Remove the directory we gave to tests to have a clean package.
+         (add-after 'check 'remove-tmp-dir
+           (lambda* (#:key outputs #:allow-other-keys)
+             (delete-file-recursively (string-append (assoc-ref outputs "out") "/tmp"))
+             #t)))))
+    (inputs
+     `(("libcap" ,libcap)))
+    (native-inputs
+     `(("python-2" ,python-2)
+       ("util-linux" ,util-linux)))
+    (home-page "https://github.com/projectatomic/bubblewrap")
+    (synopsis "Unprivileged sandboxing tool")
+    (description "Bubblewrap is aimed at running applications in a sandbox,
+restricting their access to parts of the operating system or user data such as
+the home directory.  Bubblewrap always creates a new mount namespace, and the
+user can specify exactly what parts of the file system should be made visible
+in the sandbox.  These directories are mounted with the @code{nodev} option
+by default and can be made read-only.")
+    (license license:lgpl2.0+)))
+
+(define-public bochs
+  (package
+    (name "bochs")
+    (version "2.6.9")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "https://sourceforge.net/projects/bochs/files/bochs/"
+                           version "/bochs-" version ".tar.gz"))
+       (sha256
+        (base32
+         "1379cq4cnfprhw8mgh60i0q9j8fz8d7n3d5fnn2g9fdiv5znfnzf"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:tests? #f)) ; No tests exist
+    (inputs
+     `(("libxrandr" ,libxrandr)))
+    (home-page "http://bochs.sourceforge.net/")
+    (synopsis "Emulator for x86 PC")
+    (description
+     "Bochs is an emulator which can emulate Intel x86 CPU, common I/O
+devices, and a custom BIOS.  It can also be compiled to emulate many different
+x86 CPUs, from early 386 to the most recent x86-64 Intel and AMD processors.
+Bochs can run most Operating Systems inside the emulation including Linux,
+DOS or Microsoft Windows.")
+    (license license:lgpl2.0+)))
+
+(define-public xen
+  (package
+    (name "xen")
+    (version "4.11.1")
+    (source (origin
+              (method git-fetch)
+              (uri (git-reference
+                    (url "git://xenbits.xenproject.org/xen.git")
+                    (commit (string-append "RELEASE-" version))))
+              (file-name (git-file-name name version))
+              (sha256
+               (base32
+                "1wv1hyfii14vi9lfjmnv07h2gpm3b7kvh2p55f4yy2b40simksgk"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:configure-flags
+       (list "--enable-rpath"
+             "--disable-qemu-traditional" ; It tries to do "git clone"
+             "--disable-rombios" ; would try to "git clone" via etherboot.
+             ;; TODO: Re-enable stubdom (it's "more secure" to use it).
+             "--disable-stubdom" ; tries to "git clone" old patched newlib.
+             (string-append "--with-initddir="
+                            (assoc-ref %outputs "out")
+                            "/etc/init.d")
+             (string-append "--with-system-qemu="
+                            (assoc-ref %build-inputs "qemu")
+                            "/bin/qemu-system-i386")
+             (string-append "--with-system-seabios="
+                            (assoc-ref %build-inputs "seabios")
+                            "/share/firmware/bios.bin")
+             (string-append "--with-system-ovmf="
+                            (assoc-ref %build-inputs "ovmf")
+                            "/share/firmware/ovmf_ia32.bin"))
+       #:make-flags (list "-j" "1"
+                          "XEN_BUILD_DATE=Thu Jan  1 01:00:01 CET 1970"
+                          "XEN_BUILD_TIME=01:00:01"
+                          "XEN_BUILD_HOST="
+                          "ETHERBOOT_NICS="
+                          "SMBIOS_REL_DATE=01/01/1970"
+                          "VGABIOS_REL_DATE=01 Jan 1970"
+                          ; QEMU_TRADITIONAL_LOC
+                          ; QEMU_UPSTREAM_LOC
+                          "SYSCONFIG_DIR=/tmp/etc/default"
+                          (string-append "BASH_COMPLETION_DIR="
+                                         (assoc-ref %outputs "out")
+                                         "/etc/bash_completion.d")
+                          (string-append "BOOT_DIR="
+                                         (assoc-ref %outputs "out")
+                                         "/boot")
+                          (string-append "DEBUG_DIR="
+                                         (assoc-ref %outputs "out")
+                                         "/lib/debug")
+                          (string-append "EFI_DIR="
+                                         (assoc-ref %outputs "out")
+                                         "/lib/efi") ; TODO lib64 ?
+                          "MINIOS_UPSTREAM_URL="
+                          ;(string-append "DISTDIR="
+                          ;               (assoc-ref %outputs "out"))
+)
+       #:test-target "test"
+       #:phases
+       (modify-phases %standard-phases
+        (add-after 'unpack 'unpack-mini-os
+          (lambda* (#:key inputs #:allow-other-keys)
+            (copy-recursively (assoc-ref inputs "mini-os") "extras/mini-os")
+            #t))
+        (add-after 'unpack-mini-os 'patch
+          (lambda* (#:key inputs outputs #:allow-other-keys)
+            (substitute* "tools/firmware/Rules.mk"
+             (("override XEN_TARGET_ARCH = x86_32")
+              (string-append "override XEN_TARGET_ARCH = x86_32
+override CC = " (assoc-ref inputs "cross-gcc") "/bin/i686-linux-gnu-gcc"))
+             (("^CFLAGS =$")
+              (string-append "CFLAGS=-I" (assoc-ref inputs "cross-libc")
+                             "/include\n")))
+            (substitute* "config/x86_32.mk"
+             (("CFLAGS += -m32 -march=i686")
+              (string-append "CFLAGS += -march=i686 -I"
+                             (assoc-ref inputs "cross-libc")
+                             "/include")))
+            ;; /var is not in /gnu/store , so don't try to create it.
+            (substitute* '("tools/Makefile"
+                           "tools/xenstore/Makefile"
+                           "tools/xenpaging/Makefile")
+             (("\\$\\(INSTALL_DIR\\) .*XEN_(DUMP|LOG|RUN|LIB|PAGING)_DIR.*")
+              "\n")
+             (("\\$\\(INSTALL_DIR\\) .*XEN_(RUN|LIB)_STORED.*")
+              "\n"))
+            ;; Prevent xen from creating /etc .
+            (substitute* "tools/examples/Makefile"
+             ((" install-readmes") "")
+             ((" install-configs") ""))
+            ;; Set rpath.
+            (substitute* "tools/pygrub/setup.py"
+             (("library_dirs =")
+              ; TODO: extra_link_args = ['-Wl,-rpath=/opt/foo'],
+              (string-append "runtime_library_dirs = ['"
+                             (assoc-ref outputs "out")
+                             "/lib'],\nlibrary_dirs =")))
+            #t))
+        (add-before 'configure 'patch-xen-script-directory
+          (lambda* (#:key outputs #:allow-other-keys)
+            (substitute* '("configure"
+                           "tools/configure"
+                           "docs/configure")
+             (("XEN_SCRIPT_DIR=.*")
+              (string-append "XEN_SCRIPT_DIR="
+                             (assoc-ref outputs "out")
+                             "/etc/xen/scripts")))
+            #t))
+        (add-before 'configure 'set-environment-up
+          (lambda* (#:key make-flags #:allow-other-keys)
+             (define (cross? x)
+               (string-contains x "cross-i686-linux"))
+             (define (filter-environment! filter-predicate
+                                          environment-variable-names)
+               (for-each
+                (lambda (env-name)
+                  (let* ((env-value (getenv env-name))
+                         (search-path (search-path-as-string->list env-value))
+                         (new-search-path (filter filter-predicate
+                                                  search-path))
+                         (new-env-value (list->search-path-as-string
+                                         new-search-path ":")))
+                    (setenv env-name new-env-value)))
+                environment-variable-names))
+             (setenv "CROSS_C_INCLUDE_PATH" (getenv "C_INCLUDE_PATH"))
+             (setenv "CROSS_CPLUS_INCLUDE_PATH" (getenv "CPLUS_INCLUDE_PATH"))
+             (setenv "CROSS_LIBRARY_PATH" (getenv "LIBRARY_PATH"))
+             (filter-environment! cross?
+              '("CROSS_C_INCLUDE_PATH" "CROSS_CPLUS_INCLUDE_PATH"
+                "CROSS_LIBRARY_PATH"))
+             (filter-environment! (lambda (e) (not (cross? e)))
+              '("C_INCLUDE_PATH" "CPLUS_INCLUDE_PATH"
+                "LIBRARY_PATH"))
+             ;; Guix tries to be helpful and automatically adds
+             ;; mini-os-git-checkout/include to the include path,
+             ;; but actually we don't want it to be there (yet).
+             (filter-environment! (lambda (e)
+                                    (not
+                                     (string-contains e
+                                      "mini-os-git-checkout")))
+              '("C_INCLUDE_PATH" "CPLUS_INCLUDE_PATH"
+                "LIBRARY_PATH"))
+            (setenv "EFI_VENDOR" "guix")
+             #t))
+        (replace 'build
+          (lambda* (#:key make-flags #:allow-other-keys)
+            (apply invoke "make" "world" make-flags))))))
+    (inputs
+     `(("acpica" ,acpica) ; TODO: patch iasl invocation.
+       ("bridge-utils" ,bridge-utils) ; TODO: patch invocations.
+       ("glib" ,glib)
+       ("iproute" ,iproute) ; TODO: patch invocations.
+       ("libaio" ,libaio)
+       ("libx11" ,libx11)
+       ("libyajl" ,libyajl)
+       ("ncurses" ,ncurses)
+       ("openssl" ,openssl)
+       ("ovmf" ,ovmf)
+       ("pixman" ,pixman)
+       ("qemu" ,qemu-minimal)
+       ("seabios" ,seabios)
+       ("util-linux" ,util-linux) ; uuid
+       ; TODO: ocaml-findlib, ocaml-nox.
+       ("xz" ,xz) ; for liblzma
+       ("zlib" ,zlib)))
+    (native-inputs
+     `(("dev86" ,dev86)
+       ("bison" ,bison)
+       ("cmake" ,cmake)
+       ("figlet" ,figlet)
+       ("flex" ,flex)
+       ("gettext" ,gettext-minimal)
+       ("libnl" ,libnl)
+       ("mini-os"
+       ,(origin
+         (method git-fetch)
+         (uri (git-reference
+               (url "http://xenbits.xen.org/git-http/mini-os.git")
+               (commit (string-append "xen-RELEASE-" version))))
+         (sha256
+          (base32
+           "1i8pcl19n60i2m9vlg79q3nknpj209c9ic5x10wxaicx45kc107f"))
+         (file-name "mini-os-git-checkout")))
+       ("perl" ,perl)
+       ; TODO: markdown
+       ("pkg-config" ,pkg-config)
+       ("python" ,python-2)
+       ("wget" ,wget)
+       ("cross-gcc" ,(cross-gcc "i686-linux-gnu"
+                                #:xbinutils (cross-binutils "i686-linux-gnu")
+                                #:libc (cross-libc "i686-linux-gnu")))
+       ("cross-libc" ,(cross-libc "i686-linux-gnu")) ; header files
+       ("cross-libc-static" ,(cross-libc "i686-linux-gnu") "static")))
+    (home-page "https://xenproject.org/")
+    (synopsis "Xen Virtual Machine Monitor")
+    (description "This package provides the Xen Virtual Machine Monitor
+which is a hypervisor.")
+    ;; TODO: Some files are licensed differently.  List those.
+    (license license:gpl2)
+    (supported-systems '("i686-linux" "x86_64-linux" "armhf-linux"))))
