@@ -20,7 +20,7 @@
 (define-module (test-cpan)
   #:use-module (guix import cpan)
   #:use-module (guix base32)
-  #:use-module (guix hash)
+  #:use-module (gcrypt hash)
   #:use-module (guix tests)
   #:use-module (guix grafts)
   #:use-module (srfi srfi-64)
@@ -97,7 +97,7 @@
                    ('propagated-inputs
                     ('quasiquote
                      (("perl-test-script" ('unquote 'perl-test-script)))))
-                   ('home-page "http://search.cpan.org/dist/Foo-Bar/")
+                   ('home-page "https://metacpan.org/release/Foo-Bar")
                    ('synopsis "Fizzle Fuzz")
                    ('description 'fill-in-yourself!)
                    ('license 'perl-license))
