@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2013, 2014, 2015, 2016 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2013, 2014, 2015, 2016, 2017 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -189,6 +189,10 @@ Second line" 24))
   (string->duration "1m")
   (string->duration "30d"))
 
+(test-equal "duration, 2 hours"
+  7200
+  (time-second (string->duration "2h")))
+
 (test-equal "duration, 1 second"
   (make-time time-duration 0 1)
   (string->duration "1s"))
@@ -248,9 +252,12 @@ Second line" 24))
                             (lambda ()
                               (show-manifest-transaction store m t)))))
            (string-match "guile\t1.8.8 -> 2.0.9"
-                         (with-fluids ((%default-port-encoding "ISO-8859-1"))
-                           (with-error-to-string
-                            (lambda ()
-                              (show-manifest-transaction store m t)))))))))
+                         (with-error-to-string
+                           (lambda ()
+                             ;; In Guile 2.2, %DEFAULT-PORT-ENCODING doesn't
+                             ;; influence the encoding of string ports.
+                             (set-port-encoding! (current-error-port)
+                                                 "ISO-8859-1")
+                             (show-manifest-transaction store m t))))))))
 
 (test-end "ui")
